

compute_quants <- function(data, thresholds,
                           ncores=1, quantile_dims=c('syear', 'ensemble'),
                           probs_dims=list('ensemble', 'bin'),
                           split_factor=1, na.rm=FALSE) {

  ## TODO: Adapt to the case where the forecast probability bins need
  ## to be computed. The quantiles should be the hcst quantiles, and then
  ## c2p() can be used to compute fcst probabilities for most likely terciles
  ## map.

  if (na.rm == FALSE) {
    get_quantiles <- function(x, t) {
      if (any(is.na(x))) {
	rep(NA, length(t))
      } else {
	quantile(as.vector(x), t, na.rm = FALSE)
      }
    }
  } else {
    get_quantiles <- function(x, t) {
      quantile(as.vector(x), t, na.rm = TRUE)
    }
  }
                            
  quantiles <- Apply(data,
                     target_dims = quantile_dims,
		     function(x, t) {get_quantiles(as.vector(x), thresholds)},
                     output_dims = "bin",
                     ncores = ncores, 
                     split_factor = split_factor)[[1]]
  
  return(quantiles)
}

compute_probs <- function(data, quantiles,
                          ncores=1, quantile_dims=c('syear', 'ensemble'),
                          probs_dims=list('ensemble', 'bin'),
                          split_factor=1, na.rm=FALSE) {

  if (na.rm == FALSE) {
    c2p <- function(x, t) {
      # If the array contains any NA values, return NA
      if (any(is.na(x))) {
	rep(NA, dim(t)[['bin']] + 1)
      } else {
       colMeans(convert2prob(as.vector(x), threshold = as.vector(t)))
      }
    }
  } else {
    c2p <- function(x, t) {
      if (any(!is.na(x))) { # If the array contains some non-NA values
        colMeans(convert2prob(as.vector(x), threshold = as.vector(t)))
      } else { # If the array contains NAs only
	rep(NA, dim(t)[['bin']] + 1) # vector with as many NAs as prob bins.
      }
    }
  }
  
  probs <- Apply(data = list(x = data, t = quantiles),
                 target_dims = probs_dims,
                 c2p,
                 output_dims = "bin",
                 split_factor = split_factor,
                 ncores = ncores)[[1]]

  return(probs)
}     
